// Board.cpp

// Copyright (c) 2002 By Spencer T. Parkin

#include <GL/glut.h>
#include <math.h>
#include "Marble.h"
#include "Board.h"
#include "Move.h"
#include "Main.h"

int HexagramTemplate[17][17] =
{
	{0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{3, 3, 3, 3, 1, 1, 1, 1, 1, 4, 4, 4, 4, 0, 0, 0, 0},
	{0, 3, 3, 3, 1, 1, 1, 1, 1, 1, 4, 4, 4, 0, 0, 0, 0},
	{0, 0, 3, 3, 1, 1, 1, 1, 1, 1, 1, 4, 4, 0, 0, 0, 0},
	{0, 0, 0, 3, 1, 1, 1, 1, 1, 1, 1, 1, 4, 0, 0, 0, 0},
	{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
	{0, 0, 0, 0, 5, 1, 1, 1, 1, 1, 1, 1, 1, 6, 0, 0, 0},
	{0, 0, 0, 0, 5, 5, 1, 1, 1, 1, 1, 1, 1, 6, 6, 0, 0},
	{0, 0, 0, 0, 5, 5, 5, 1, 1, 1, 1, 1, 1, 6, 6, 6, 0},
	{0, 0, 0, 0, 5, 5, 5, 5, 1, 1, 1, 1, 1, 6, 6, 6, 6},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0},
};

Node *NodeNetworkMatrix[17][17];
Node GameBoard[121];

Node::Node()
{
	Calc = 1;
	Mbl = 0;
	Jump = 0;
	Search = 1;
}

Node::~Node()
{
}

void Node::CalcBoardGeometry(GLfloat X, GLfloat Y, GLfloat Z)
{
	this->X = X;
	this->Y = Y;
	this->Z = Z;

	Calc = 0;

	int i;
	static GLfloat Angle;

	for(i = 0; i < NODE_MOVES; i++)
	{
		if(Move[i] && Move[i]->Calc)
		{
			Angle = (GLfloat(i) / GLfloat(NODE_MOVES)) * 2.f * 3.14159263f;
			Move[i]->CalcBoardGeometry(X + cos(Angle), Y + sin(Angle), Z);
		}
	}
}

void MakeGameBoard(void)
{
	int i, j;
	Node *N = GameBoard;

	// Populate the node matrix.
	for(i = 0; i < 17; i++)
	{
		for(j = 0; j < 17; j++)
		{
			if(HexagramTemplate[i][j])
			{
				N->Color = (MarbleColor)(HexagramTemplate[i][j] - 1);
				NodeNetworkMatrix[i][j] = N++;
			}
			else
			{
				NodeNetworkMatrix[i][j] = 0;
			}
		}
	}

	// Link the node-network together.
	for(i = 0; i < 17; i++)
	{
		for(j = 0; j < 17; j++)
		{
			N = NodeNetworkMatrix[i][j];

			if(N)
			{
				int igz, jgz;
				int ils, jls;

				igz = i > 0;
				jgz = j > 0;
				ils = i < 16;
				jls = j < 16;

				

				N->Move[0] = jls        ? NodeNetworkMatrix[i    ][j + 1] : 0;
				N->Move[1] = igz        ? NodeNetworkMatrix[i - 1][j    ] : 0;
				N->Move[2] = igz && jgz ? NodeNetworkMatrix[i - 1][j - 1] : 0;
				N->Move[3] = jgz        ? NodeNetworkMatrix[i    ][j - 1] : 0;
				N->Move[4] = ils        ? NodeNetworkMatrix[i + 1][j    ] : 0;
				N->Move[5] = ils && jls ? NodeNetworkMatrix[i + 1][j + 1] : 0;
			}
		}
	}

	// Calculate the hexagram geometry.
	NodeNetworkMatrix[8][8]->CalcBoardGeometry(0.f, 0.f, 0.f);
}

void ResetGameBoard(void)
{
	Marble *Mbl = GamePeices;
	int i;
	Node *N;

	// We assume the game board has been made.
	for(i = 0; i < 121; i++)
	{
		N = GameBoard + i;
		
		if(N->Color != NULL_MARBLE)
		{
			Mbl->Color = N->Color;
			N->Mbl = Mbl++;
		}
	}
}

void RenderGameBoard(GLenum Mode)
{
	int i, j;
	Node *N;

	if(Mode == GL_RENDER)
	{
		for(i = 0; i < 121; i++)
		{
			N = GameBoard + i;

			for(j = 0; j < NODE_MOVES; j++)
			{
				N->DrawLine[j] = 1;
			}
		}

		glBegin(GL_LINES);

		for(i = 0; i < 121; i++)
		{
			N = GameBoard + i;

			for(j = 0; j < NODE_MOVES; j++)
			{
				if(N->Move[j] && N->DrawLine[j])
				{
					SpecifyMarbleColor(N->Color ? N->Color : N->Move[j]->Color);

					glVertex3fv(&N->X);
					glVertex3fv(&N->Move[j]->X);

					N->Move[j]->DrawLine[(j + 3) % NODE_MOVES] = 0;
				}
			}
		}

		glEnd();
	}

	for(i = 0; i < 121; i++)
	{
		N = GameBoard + i;

		if(Mode == GL_SELECT)
		{
			glPushName(i);
		}

		if(N->Mbl)
		{
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();

			if(N == JumpingNode)
			{
				glTranslatef(
						JumpAxis[0][0],
						JumpAxis[0][1],
						JumpAxis[0][2]);
								
				glRotatef(JumpAngle * 180.f / PI,
						JumpAxis[1][0],
						JumpAxis[1][1],
						JumpAxis[1][2]);

				glTranslatef(
						N->X - JumpAxis[0][0],
						N->Y - JumpAxis[0][1],
						N->Z - JumpAxis[0][2]);
			}
			else
			{
				glTranslatef(N->X, N->Y, N->Z);
			}

			N->Mbl->Render(Mode);

			glPopMatrix();
		}
		else if(Mode == GL_SELECT)
		{
			glBegin(GL_TRIANGLE_FAN);
			glVertex3fv(&N->X);

			GLfloat Angle;

			for(j = 0; j <= NODE_MOVES; j++)
			{
				Angle = (GLfloat(j) / GLfloat(NODE_MOVES)) * 2.f * 3.14159263f;
				glVertex3f(N->X + 0.3f * cos(Angle), N->Y + 0.3f * sin(Angle), 0.f);
			}

			glEnd();
		}
	}
}

Node *PickNode(int X, int Y)
{
	GLuint SelectBuffer[512];
	GLint SelectHits;
	GLint Viewport[4];

	glGetIntegerv(GL_VIEWPORT, Viewport);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluPickMatrix(GLdouble(X), GLdouble(Viewport[3] - Y), 3, 3, Viewport);
	gluPerspective(60.f, GLfloat(Viewport[2]) / GLfloat(Viewport[3]), 1.f, 1000.f);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glTranslatef(0.f, 0.f, Zoom);
	glRotatef(Xrot, 1.f, 0.f, 0.f);
	glRotatef(Yrot, 0.f, 1.f, 0.f);
	glRotatef(Zrot, 0.f, 0.f, 1.f);

	glSelectBuffer(512, SelectBuffer);
	(void)glRenderMode(GL_SELECT);

	glInitNames();
	glPushName(-1);

	RenderGameBoard(GL_SELECT);

	glFlush();
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);

	SelectHits = glRenderMode(GL_RENDER);

	int i;
	GLuint *HitData = SelectBuffer;
	GLuint NumNames;
//	GLfloat Zmin, Zmax;
	Node *N = 0;

	for(i = 0; i < SelectHits; i++)
	{
		NumNames = *HitData++;

//		Zmin = *HitData++ / 0x7FFFFFFF;
//		Zmax = *HitData++ / 0x7FFFFFFF;

		HitData += NumNames + 2 - 1;

		N = GameBoard + *HitData++;
	}

	return(N);
}

// Board.cpp