// Main.cpp

// Copyright (c) 2002 By Spencer T. Parkin

#include <GL/glut.h>
#include "Marble.h"
#include "Board.h"
#include "Move.h"
#include "Main.h"

int Xmouse, Ymouse;

GLfloat Zoom = -14.f;
GLfloat Xrot = 0.f;
GLfloat Yrot = 0.f;
GLfloat Zrot = 0.f;

Node *SelectedNode = 0;

main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
	glutInitWindowPosition(100, 100);
	glutInitWindowSize(500, 500);
	glutCreateWindow(argv[0]);

	MakeGameBoard();
	ResetGameBoard();

	glClearColor(0.f, 0.f, 0.f, 0.f);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT0);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glLineWidth(1.3f);
	glShadeModel(GL_SMOOTH);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	GLfloat MatSpec[] = {1.0f, 1.0f, 1.0f, 1.f};
	GLfloat MatShine[] = {50.f};
	GLfloat DiffuseLight[] = {0.6f, 0.6f, 0.6f, 1.f};
	GLfloat LightPos[] = {1.f, 1.f, 1.f, 0.f};

	glMaterialfv(GL_FRONT, GL_SPECULAR, MatSpec);
	glMaterialfv(GL_FRONT, GL_SHININESS, MatShine);

	glLightfv(GL_LIGHT0, GL_DIFFUSE, DiffuseLight);
	glLightfv(GL_LIGHT0, GL_POSITION, LightPos);

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutIdleFunc(idle);

	glutMainLoop();

	return(0);
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glTranslatef(0.f, 0.f, Zoom);

	glRotatef(Xrot, 1.f, 0.f, 0.f);
	glRotatef(Yrot, 0.f, 1.f, 0.f);
	glRotatef(Zrot, 0.f, 0.f, 1.f);

	RenderGameBoard(GL_RENDER);

	glPopMatrix();

	glutSwapBuffers();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.f, GLfloat(w) / GLfloat(h), 1.f, 1000.f);
	glMatrixMode(GL_MODELVIEW);
}

void mouse(int button, int state, int x, int y)
{
	switch(state)
	{
		case GLUT_UP:
		{
			break;
		}
		case GLUT_DOWN:
		{
			Xmouse = x;
			Ymouse = y;

			switch(button)
			{
				case GLUT_LEFT_BUTTON:
				{
					CompletePendingMove();

					Node *N = PickNode(x, y);

					if(!N || N == SelectedNode)
					{
						SelectedNode = 0;
					}
					else if(MakeMove(SelectedNode, N))
					{
						JumpingNode = SelectedNode;
						SelectedNode = 0;
					}
					else
					{
						SelectedNode = N;
					}

					break;
				}
				case GLUT_RIGHT_BUTTON:
				{
					break;
				}
			}

			break;
		}
	}

	glutPostRedisplay();
}

void motion(int x, int y)
{
	Xrot += GLfloat(y - Ymouse);
	Zrot += GLfloat(x - Xmouse);

	Ymouse = y;
	Xmouse = x;

	glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
		case '[':
		{
			Zoom -= 1.2f;
			break;
		}
		case ']':
		{
			Zoom += 1.2f;
			break;
		}
	}

	glutPostRedisplay();
}

void idle(void)
{
	AnimateMovingMarbles();

	glutPostRedisplay();
}

// Main.cpp