// Marble.cpp

// Copyright (c) 2002 By Spencer T. Parkin

#include <GL/glut.h>
#include <math.h>
#include "Marble.h"
#include "Board.h"
#include "Main.h"

Marble GamePeices[GAME_PEICES];

#define U	0.525731112119133606f
#define V	0.850650808352039932f

GLfloat IcosVertices[12][4] =
{
	{-U, 0.f, V}, {U, 0.f, V}, {-U, 0.f, -V}, {U, 0.f, -V},
	{0.f, V, U}, {0.f, V, -U}, {0.f, -V, U}, {0.f, -V, -U},
	{V, U, 0.f}, {-V, U, 0.f}, {V, -U, 0.f}, {-V, -U, 0.f},
};

int IcosFaces[20][3] =
{
	{0, 4, 1}, {0, 9, 4}, {9, 5, 4}, {4, 5, 8}, {4, 8, 1},
	{8, 10, 1}, {8, 3, 10}, {5, 3, 8}, {5, 2, 3}, {2, 7, 3},
	{7, 10, 3}, {7, 6, 10}, {7, 11, 6}, {11, 0, 6}, {0, 1, 6},
	{6, 1, 10}, {9, 0, 11}, {9, 11, 2}, {9, 2, 5}, {7, 2, 11},
};

Marble::Marble()
{
}

Marble::~Marble()
{
}

void Marble::Render(GLenum Mode)
{
	SpecifyMarbleColor(Color);

	if(Mode == GL_SELECT)
	{
		//glLoadName(
	}
	else if(Mode == GL_RENDER)
	{
		glEnable(GL_LIGHTING);

		static GLfloat AmbientLight[4];

		if(SelectedNode && SelectedNode->Mbl == this)
		{
			AmbientLight[0] = 1.f;
			AmbientLight[1] = 1.f;
			AmbientLight[2] = 1.f;
			AmbientLight[3] = 1.f;
		}
		else
		{
			switch(Color)
			{
				case RED_MARBLE:
				{
					AmbientLight[0] = 1.f;
					AmbientLight[1] = 0.f;
					AmbientLight[2] = 0.f;
					AmbientLight[3] = 1.f;

					break;
				}
				case GREEN_MARBLE:
				{
					AmbientLight[0] = 0.f;
					AmbientLight[1] = 1.f;
					AmbientLight[2] = 0.f;
					AmbientLight[3] = 1.f;

					break;
				}
				case BLUE_MARBLE:
				{
					AmbientLight[0] = 0.f;
					AmbientLight[1] = 0.f;
					AmbientLight[2] = 1.f;
					AmbientLight[3] = 1.f;

					break;
				}
				case MAGENTA_MARBLE:
				{
					AmbientLight[0] = 1.f;
					AmbientLight[1] = 0.f;
					AmbientLight[2] = 1.f;
					AmbientLight[3] = 1.f;

					break;
				}
				case CYAN_MARBLE:
				{
					AmbientLight[0] = 1.f;
					AmbientLight[1] = 1.f;
					AmbientLight[2] = 0.f;
					AmbientLight[3] = 1.f;

					break;
				}
				case YELLOW_MARBLE:
				{
					AmbientLight[0] = 0.f;
					AmbientLight[1] = 1.f;
					AmbientLight[2] = 1.f;
					AmbientLight[3] = 1.f;

					break;
				}
				default:
				{
					AmbientLight[0] = 1.f;
					AmbientLight[1] = 1.f;
					AmbientLight[2] = 1.f;
					AmbientLight[3] = 1.f;

					break;
				}
			}
		}

		glLightfv(GL_LIGHT0, GL_AMBIENT, AmbientLight);
	}

	glBegin(GL_TRIANGLES);

	int i;

	for(i = 0; i < 20; i++)
	{
		DrawTriangle(IcosVertices[IcosFaces[i][0]],
					 IcosVertices[IcosFaces[i][1]],
					 IcosVertices[IcosFaces[i][2]], 1);
	}

	glEnd();

	if(Mode == GL_RENDER)
	{
		glDisable(GL_LIGHTING);
	}
}

void DrawTriangle(GLfloat *v0, GLfloat *v1, GLfloat *v2, int SubDivDepth)
{
	int i;

	if(SubDivDepth > 0)
	{
		GLfloat v01[3], v12[3], v20[3];

		for(i = 0; i < 3; i++)
		{
			v01[i] = v0[i] + v1[i];
			v12[i] = v1[i] + v2[i];
			v20[i] = v2[i] + v0[i];
		}

		Normalize(v01);
		Normalize(v12);
		Normalize(v20);

		DrawTriangle(v0, v01, v20, SubDivDepth - 1);
		DrawTriangle(v1, v12, v01, SubDivDepth - 1);
		DrawTriangle(v2, v20, v12, SubDivDepth - 1);
		DrawTriangle(v01, v12, v20, SubDivDepth - 1);
	}
	else
	{
		GLfloat tri[3][3];

		for(i = 0; i < 3; i++)
		{
			tri[0][i] = v0[i] * 0.35f;
			tri[1][i] = v1[i] * 0.35f;
			tri[2][i] = v2[i] * 0.35f;
		}

		glNormal3fv(v0);
		glVertex3fv(tri[0]);

		glNormal3fv(v1);
		glVertex3fv(tri[1]);

		glNormal3fv(v2);
		glVertex3fv(tri[2]);
	}
}

void Normalize(GLfloat *Vec)
{
	GLfloat Scalar = 1.f / sqrt(
					Vec[0] * Vec[0] +
					Vec[1] * Vec[1] +
					Vec[2] * Vec[2]);

	Vec[0] *= Scalar;
	Vec[1] *= Scalar;
	Vec[2] *= Scalar;
}

void SpecifyMarbleColor(MarbleColor Color)
{
	switch(Color)
	{
		case RED_MARBLE:		glColor3f(1.f, 0.f, 0.f); break;
		case GREEN_MARBLE:		glColor3f(0.f, 1.f, 0.f); break;
		case BLUE_MARBLE:		glColor3f(0.f, 0.f, 1.f); break;
		case MAGENTA_MARBLE:	glColor3f(1.f, 0.f, 1.f); break;
		case CYAN_MARBLE:		glColor3f(1.f, 1.f, 0.f); break;
		case YELLOW_MARBLE:		glColor3f(0.f, 1.f, 1.f); break;
		case NULL_MARBLE:
		default:
		{
			glColor3f(1.f, 1.f, 1.f);
		}
	}
}


// Marble.cpp