// Move.cpp

// Copyright (c) 2002 By Spencer T. Parkin

#include <GL/glut.h>
#include <math.h>
#include "Marble.h"
#include "Board.h"
#include "Move.h"

Node *JumpingNode = 0;
int CalcJumpAxisAngle = 1;
GLfloat JumpAxis[2][3];
GLfloat JumpAngle;

Node *JumpMarble(Node *N)
{
	if(N->Jump)
	{
		N->Jump->Mbl = N->Mbl;
		N->Mbl = 0;	
	}

	return(N->Jump);
}

void CompletePendingMove(void)
{
	while(JumpingNode)
	{
		JumpingNode = JumpMarble(JumpingNode);
	}

	CalcJumpAxisAngle = 1;
}

void AnimateMovingMarbles(void)
{
	for(;;)
	{
		if(JumpingNode)
		{		
			if(CalcJumpAxisAngle && JumpingNode->Jump)
			{
				CalcJumpAxisAngle = 0;

				JumpAngle = JUMP_ANGLE;

				JumpAxis[0][0] = (JumpingNode->X + JumpingNode->Jump->X) * 0.5f;
				JumpAxis[0][1] = (JumpingNode->Y + JumpingNode->Jump->Y) * 0.5f;
				JumpAxis[0][2] = (JumpingNode->Z + JumpingNode->Jump->Z) * 0.5f;

				GLfloat Xdelta = JumpingNode->Jump->X - JumpingNode->X;
				GLfloat Ydelta = JumpingNode->Jump->Y - JumpingNode->Y;
				GLfloat Angle = atan(Ydelta / Xdelta) + PI / 2.f;

				JumpAxis[1][0] = cos(Angle);
				JumpAxis[1][1] = sin(Angle);
				JumpAxis[1][2] = 0.f;			
			}
			else if(JumpAngle + JUMP_ANGLE < PI)
			{
				JumpAngle += JUMP_ANGLE;
			}
			else
			{
				CalcJumpAxisAngle = 1;

				JumpingNode = JumpMarble(JumpingNode);

				continue;
			}
		}
		
		break;
	}
}

int MakeMove(Node *Src, Node *Dst)
{
	if(Src && Dst)
	{
		if(Src->Mbl && !Dst->Mbl)
		{
			return(FindMove(Src, Dst, 1, 1));
		}
	}

	return(0);
}

static int FindMove(Node *Src, Node *Dst, int Jump1, int Jump2)
{
	int MoveFound = (Src == Dst);

	if((Jump2 || Jump1) && !MoveFound)
	{
		Src->Search = 0;

		int i;

		// Perform a depth-first search...
		for(i = 0; i < NODE_MOVES && !MoveFound; i++)
		{
			if(Src->Move[i])
			{
				if(Src->Move[i]->Mbl)
				{
					if(Src->Move[i]->Move[i] && !Src->Move[i]->Move[i]->Mbl)
					{
						if(Src->Move[i]->Move[i]->Search && Jump2)
						{
							MoveFound = FindMove(Src->Move[i]->Move[i], Dst, 0, 1);
						}
					}
				}
				else if(Src->Move[i]->Search && Jump1)
				{
					MoveFound = FindMove(Src->Move[i], Dst, 0, 0);
				}
			}
		}

		// Chain moves via tail-recursion.
		if(MoveFound)
		{
			if(Src->Move[i - 1]->Mbl)
			{
				Src->Jump = Src->Move[i - 1]->Move[i - 1];
			}
			else
			{
				Src->Jump = Src->Move[i - 1];
			}
		}

		Src->Search = 1;
	}

	if(MoveFound)
	{
		Dst->Jump = 0;
	}

	return(MoveFound);
}

// Move.cpp